/*
 * Device driver for PI PCI stage controller
 * Released under GPL
 * (c) Matt Clark January 2003
 * 
 *
 */


/*
 * As of January 2003 PI do not seem to have their own
 * PCI vendor id of device id and are using the defaults 
 * their PCI<-> local bus bridge which is the TI PCI2040
 * so they have vendor id of TI (0x104c) and the device id
 * of PCI2040 Bridge		0xac60. 
 *
 */


#include <linux/ioctl.h>
#include <asm/types.h>
 
/* incidentals */
#define TRUE	1
#define FALSE	0

#define PI_MAGIC		106
#define PI_STAGE_HARDRESET 	_IO(PI_MAGIC,0)
#define PI_GET_LOOPS	 	_IO(PI_MAGIC,1)
#define PI_RESET_BOARD		_IO(PI_MAGIC,2)
#define PI_SET_0	 	_IOWR(PI_MAGIC,10,0)
#define PI_SET_CMD	 	_IOWR(PI_MAGIC,10,0)
#define PI_SET_1	 	_IOWR(PI_MAGIC,11,0)
#define PI_SET_2	 	_IOWR(PI_MAGIC,12,0)
#define PI_SET_3	 	_IOWR(PI_MAGIC,13,0)
#define PI_GET_1	 	_IOWR(PI_MAGIC,20,0)
#define PI_GET_2	 	_IOWR(PI_MAGIC,21,2)
#define PI_BLANK_READ		_IOWR(PI_MAGIC,30,0) // for debug only
#define PI_SET_GET_1		_IOWR(PI_MAGIC,31,0)
#define PI_SET_GET_2		_IOWR(PI_MAGIC,32,0)

// driver error codes - you should never ever see these
#define PI_NO_ERROR		0
#define PI_LOOPED_OUT		1	// shouldn't happen but if it does beware PI_LOOPED_OUT_FATAL
#define PI_LOOPED_OUT_FATAL	2	// unable to get a ready for IO signal - board crashed
#define PI_RESET_FAIL		4	// board reset resulted in the wrong values - try longer timing in pi_board_reset

/* standard header files */
#ifdef MODULE
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/fs.h>
#include <linux/config.h>
#include <linux/pci.h>
#include <linux/proc_fs.h>
#include <linux/spinlock.h>
#include <linux/interrupt.h>
#include <asm/segment.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <asm/delay.h>
/* Hacks / Tweaks /Options here */

/* IDs */
#define PI_MOD_NAME	"pi_stage"
#define PI_IRQ_ID	"PI stage"
#define PI_VENDOR_ID	0x104c
#define PI_DEVICE_ID	0xac60
#define PI_GET_CHECKSUM	0xf8
#define PI_CHECKSUM	0x12345678

// yes it is a long wait (in loops on a 750MHz machine) but some commands
// are just that slow - will survey to find out which
#define PI_MAX_WAIT	200

/* name the memory regions */
#define PI_STAGE_HPICSR 0
#define PI_STAGE_CSPACE	1
#define PI_STAGE_GPBUS	2

/* now the sizes (available by probing though) */
#define PI_STAGE_HPICSR_SIZE		0x00000020 	/* the region is much bigger (4k)*/
#define PI_STAGE_CSPACE_SIZE		0x00008000 	/* 32k */
#define PI_STAGE_GPBUS_SIZE		0x000000ff	/* we only use a fraction of this (4 bytes) */



/* useful PCI configuration registers and values as used by TI PCI2040 */

#define PI_STAGE_PCIREG_CMD		0x04
#define PI_STAGE_PCIREG_CMD_IOEN	0x0001	// enable io space(s)
#define PI_STAGE_PCIREG_CMD_MEMEN	0x0002	// enable memory space(s)
#define PI_STAGE_PCIREG_CMD_MASTEN	0x0004	// enable bus mastering (hardwired to 0)
#define PI_STAGE_PCIREG_CMD_SPECIAL	0x0008	// enable special cycles (hardwired to 0) (ignores specials)
#define PI_STAGE_PCIREG_CMD_MWIEN	0x0010	// "memory write invalidate" enable (MWI) (hardwired to 0, MW)
#define PI_STAGE_PCIREG_CMD_VGAEN	0x0020	// VGA Palette snoop (hardwired to 0)
#define PI_STAGE_PCIREG_CMD_PERREN	0x0040	// Parity error response (yes=1, no=0)
#define PI_STAGE_PCIREG_CMD_STEPEN	0x0080	// Address/Data stepping (hardwired to 0 = no)
#define PI_STAGE_PCIREG_CMD_SERREN	0x0100	// System error enable (0 disable, 1 enable (allows system errors to reach PCI bus)
#define PI_STAGE_PCIREG_CMD_FBBEN	0x0200	// Fast B2B enable (hardwired to 0=disabled)
#define PI_STAGE_PCIREG_CMD_OURS	PI_STAGE_PCIREG_CMD_SERREN | PI_STAGE_PCIREG_CMD_PERREN | PI_STAGE_PCIREG_CMD_MEMEN

#define PI_STAGE_PCIREG_GPIO_SELECT	0x44
#define PI_STAGE_PCIREG_GPIO_GPIO3PIN	0x08	// enables GPIO3 interrupts (PCI2040 PCI-DSP bridge Controller Data Manual page 4-4)

#define PI_STAGE_PCIREG_GPIO_INDATA	0x45
#define PI_STAGE_PCIREG_GPIO_DIRECTION	0x46
#define PI_STAGE_PCIREG_GPIO_OUTDATA	0x47
#define PI_STAGE_PCIREG_GPIO_INTTYPE	0x48
#define PI_STAGE_PCIREG_GPIO_LOW	0x00	// INTGPIO3 (and 2) set when GPIO3(2) is low (PCI2040 PCI-DSP bridge Controller Data Manual page 4-14) 

#define PI_BOARD_RESET			0x39	// same as PI_RESET in pi_cmd.h
#define PI_GET_HOST_IO_ERROR            0xa5
/* register defines - all offsets bytes wrt to start of memory region */

#define PI_STAGE_INTEVENTSET		0x00	// 32 bits
#define PI_STAGE_INTEVENTCLEAR		0x04	// 32 bits
#define PI_STAGE_INTMASKSET		0x08	// 32 bits
#define PI_STAGE_INTMASKCLEAR		0x0c	// 32 bits
#define PI_STAGE_HPIERRORREPORT		0x10	// 16 bits
#define PI_STAGE_RESERVED1		0x12	// 16 bits
#define PI_STAGE_HPIRESETREG		0x14	// 16 bits 
#define PI_STAGE_HPIIMPLEMT		0x16	// 16 bits
#define PI_STAGE_HPIDATAWIDTH		0x18	// 16 bits
#define PI_STAGE_RESERVED2		0x20	// 16 bits


/* interrupt flags */
#define PI_STAGE_INTDSP0	0x00000001
#define PI_STAGE_INTDSP1	0x00000002
#define PI_STAGE_INTDSP2	0x00000004
#define PI_STAGE_INTDSP3	0x00000008
#define PI_STAGE_GPINT		0x04000000
#define PI_STAGE_INTGPIO2	0x08000000
#define PI_STAGE_INTGPIO3	0x10000000
#define PI_STAGE_GPERROR	0x20000000
#define PI_STAGE_HPIERROR	0x40000000
#define PI_STAGE_ENABLEINT	0x80000000

#define PI_STAGE_ALLINTS	0x7c00000f // all of them added up 

// the odd command needed by the driver directly 
// commands are usually defined in pi_cmd.h part of the library not here
#define PI_GET_INTERRUPT_AXIS	0xe1
#define PI_GET_VERSION		0x8f
#define PI_CLEAR_INTERRUPT	0xac
#define PI_GET_EVENT_STATUS	0x31
#define PI_RESET_EVENT_STATUS	0x34



// sensible functional names 
#define PI_STAGE_AXIS_INT	PI_STAGE_GPINT
#define PI_STAGE_IO_INT 	PI_STAGE_INTGPIO3

/* all comms to the board follow this structure CMD,DATA,DATA,... */
/* all comms goto the GP bus which has the following structure *
 * offset function *
 * 0 data I/O *
 * 2 CMD (I only) */

/* some stuff for the motion controller via GP bus			*/

// uncomment if you want to poll during reads (doesn't seem necessary)
//#define PI_PLAY_REALLY_SAFE

/* GP bus offsets							*/
#define PI_STAGE_GP_DATA	0
#define PI_STAGE_GP_CMD		4 // note on line 735 of WinNT40Driver/Pci2040IRQ.cpp this is 2 (shorts...)

/* GP bus commands							*/
#define PI_CMD_NOOPS			0x00	// null command for testing
#define PI_CMD_GETINTAXIS		0xe1	// command get interrupt axis 
#define PI_CMD_GETSTATUSREG		0x31	// command get event status register
#define PI_CMD_CLEARESR			0x34	// command clear (reset) event status register
#define PI_CMD_CLEARINT			0xac	// command clear interrupt


// what to do in the event of an interrupt?
#define PI_STAGE_IDLE	0	// we aren't expecting an interrupt
#define PI_STAGE_READ	1	// we are reading from the board
#define PI_STAGE_WRITE	2	// we are writing to the board

#define PI_N_EVENTS	32	// size of the ring buffer 

struct pi_stage_event_t{
	int index;
	int axis;
	unsigned long ts_high,ts_low;
	u32 event;
	};


struct pi_stage_t{
	struct pci_dev *dev; // Well.. the config space apparently needs to be probed regularly for IOs
	
	/* address for our card */
	void *	control;
	void * 	config;
	void *	gpbus;

	int	naxis;

	int 	locked;		// to enforce single openness (later..)
	int 	interrupt; 	/* our interrupt line*/
	int	ints_all;
	int	ints_io;
	int	ints_axis;
	int	ints_board;

	int	error;		// to record driver bad behaviour
	
	struct	pi_stage_event_t * events;
	int	event_head;
	int 	event_tail;
	int	event_index;
	int	async;
	struct fasync_struct    *async_queue;
	};

/* single board at the moment */
struct pi_stage_t pi_stage={

	control:	NULL,
	config:		NULL,
	gpbus:		NULL,
	locked:		FALSE,
	interrupt:	-1,
	ints_all:	0,	// all the interrupts we rec
	ints_io:	0,	// all the interesting ones
	ints_axis:	0,	// all the interesting ones
	ints_board:	0,	// all that are come from our board

	error:		PI_NO_ERROR,
	
	event_head:	0,
	event_tail:	0,
	event_index:	0,
	async:		FALSE,
	async_queue:	NULL,
	};	










#endif
